/*========================================================================
     "gpop3tst"  Gaussian output post processor 3
        - modification of .gpo data for thermo and TST calculations

             Copyright (c) 2002-2005 by A. Miyoshi, Univ. Tokyo
                                     created: Dec.   4, 2004
                                      edited: Jan.  26, 2005
                                      edited: Nov.  19, 2007
--------------------------------------------------------------------------
 usage: gpop3tst basename [-tst]
--------------------------------------------------------------------------
 - Make modification to basename.gpo according to the information provided
    in basename.mod.
 - With option '-tst' basename.tst is created.
 - Keywords allowed in .mod file are:
    gElec g(int)
    eStates n(int) {g(int) e(double)} * n
    rotSymNbr sgm(int)
    numIsomers nIso(int)
    isomStates n(int) {g(int) e(double)} * n
    isTS flag(bool)
    energyTST eTST(double)
    useSCFenergy+ZPE
    setIntRotor iVib(int) nSym(int) moiety1(s) moiety2(s) decription(s)
========================================================================*/

#include "gpoplibt.h"

/*===== main function ==================================================*/

int main(int argc, char *argv[]) {
  int i, idv, nsm, nimg, outTST = false, nes, ies, nis, iis, idtk;
  string opt, basfln, modfln, slin, key, dscr;
  vector<string> modtkn, moin;
  jobInfos jbinf;
  molecGeom molc;
  freqTable frtab;
  modInfoTST miTST;
  intRotor irot;
  eigenState elst, isst;
  ifstream inf;

  showRevision();
  if ((argc < 2) || (argc > 3))
    { cout << "usage: gpop3tst basename [-tst]\n"; return 1; }
  if (argc == 3) {
    opt = argv[2];
    if (opt != "-tst")
      { cout << "usage: gpop3tst basename [-tst]\n"; return 1; }
    else { outTST = true; }
  }
  basfln = argv[1]; modfln = basfln + ".mod";
  if (readGpo(basfln, jbinf, molc, frtab) == ERRE) { return 1; }
  nimg = frtab.countImag();
  if (nimg > 0) { miTST.isTS = true; }

  inf.open(modfln.c_str());
  if (!inf)
    { cout << "failed to open [" << modfln << "].\n"; return 1; }
  while (getLineGpop(inf, slin) == NORM) {
    modtkn = splitTokens(slin, " ");
    if (modtkn.size() < 1)
      { cout << "Wrong mod-input.\n"; inf.close(); return 1; }
    key = modtkn[0];
    if ((key != "useSCFenergy+ZPE") && (modtkn.size() < 2))
      { cout << "Wrong mod-input.\n"; inf.close(); return 1; }
    if (key == "gElec")
      { molc.gElec = atoi(modtkn[1].c_str()); }
    else if (key == "rotSymNbr")
      { molc.rotSymNbr = atoi(modtkn[1].c_str()); }
    else if (key == "numIsomers")
      { molc.numIsomers = atoi(modtkn[1].c_str()); }
    else if (key == "isTS")
      { miTST.isTS = getBlnvStr(modtkn[1]); }
    else if (key == "energyTST")
      { miTST.energyTST = atof(modtkn[1].c_str()); }
    else if (key == "useSCFenergy+ZPE") {
      miTST.energyTST = atof(jbinf.SCFenergy.c_str())
       + frtab.zpe / WAVNHRT;
    } else if (key == "setIntRotor") {
      if (modtkn.size() < 5)
        { cout << "Wrong mod-input.\n"; inf.close(); return 1; }
      idv = atoi(modtkn[1].c_str()); nsm = atoi(modtkn[2].c_str());
      moin.clear();
      for (i = 3; i < 5; i++) { moin.push_back(modtkn[i]); }
      dscr.erase();
      if (modtkn.size() > 5) { dscr = modtkn[5]; }
      irot.setFrom(idv, nsm, moin, dscr, molc, frtab);
      miTST.hndRots.push_back(irot);
    } else if (key == "eStates") {
      nes = atoi(modtkn[1].c_str());
      for (ies = 0; ies < nes; ies++) {
        idtk = ies * 2 + 2;
        elst.deg = atoi(modtkn[idtk].c_str());
        elst.energy = atof(modtkn[idtk + 1].c_str());
        miTST.eStates.push_back(elst);
      }
    } else if (key == "isomStates") {
      nis = atoi(modtkn[1].c_str());
      for (iis = 0; iis < nis; iis++) {
        idtk = iis * 2 + 2;
        isst.deg = atoi(modtkn[idtk].c_str());
        isst.energy = atof(modtkn[idtk + 1].c_str());
        miTST.isomStates.push_back(isst);
      }
    } else { cout << "Warning: Invalid key [" << key << "].\n"; }
  }
  inf.close();

  miTST.hndRots.diagnIntRotAssign(frtab);
  switch (nimg) {
   case 0: if (miTST.isTS)
      { cout << "Warning: TS flag set while no imag. freq. found.\n"; }
    break;
   case 1: if (!miTST.isTS)
      { cout << "Warning: TS flag not set while imag. freq. found.\n"; }
    break;
   default:
    cout << "Warning: More than 1 imag. freq. found.\n";
    if (!miTST.isTS)
      { cout << "Warning: TS flag not set while imag. freq. found.\n"; }
    break;
  }
  if (miTST.isTS && miTST.hndRots.isIntRotor(0)) {
    cout << "Error: TS flag set while vib#0 is hind. rot.\n"; return 1;
  }
  if (outFileGpo(basfln, jbinf, molc, frtab, miTST) == ERRE) { return 1; }
  if (outTST) {
    if (outFileTst(basfln, jbinf, molc, frtab, miTST) == ERRE) { return 1; }
  }
  return 0;
}

