/*========================================================================
     This is a part of "gpop" - Gaussian output post processor
             Copyright (c) 2002-2007 by A. Miyoshi, Univ. Tokyo
========================================================================*/

#ifndef _GPOPLIBF_H_
#define _GPOPLIBF_H_

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <cctype>
#include <cstring>
#include <cstdlib>
#include <cmath>
using namespace std;

#ifndef true
#define true (-1)
#endif
#ifndef false
#define false 0
#endif
#ifndef M_PI
#define	M_PI    3.14159265358979323846
#endif
#define GPOPREV "GPOP rev. 2007.11.19"
#define NORM 0
#define ERRE 1
#define MAXLN1L 256
#define MAXSMAT 10
#define WAVNHRT 219474.6314        // cm-1 hartree-1
#define KJMLHRT 2625.499625        // kJ/mol hartree-1
#define BIWAA2  16.8576291         // cm-1 amu A2
#define GHZWAVN 29.9792458         // GHz (cm-1)-1
#define KELWAVN 1.438775246        // K (cm-1)-1
#define JMLWAVN 11.9626565         // J mol-1 (cm-1)-1
#define RGASC   8.314472           // J K-1 mol-1
#define STDTMPR 298.15
#define RBCMMLC 1.380650276e-22    // bar K-1 cm3 molecule-1
#define CTRANS  1.879321261e+20
#define CSTR1B  (-1.151704903)
#define TSTK_H  2.083664275e+10    // kB/h
#define GAUCPRC 0.000001
#define IX  0
#define IY  1
#define IZ  2

/*----- generic library funtions ---------------------------------------*/

void showRevision();
string getBasePart(string flname);
string getExtPart(string flname);
int getLineRmCmm(istream &inStrm, string &sLine, const string &cmmSyms);
int getLine(istream &inStrm, string &sLine);
int getLineTrim(istream &inStrm, string &sLine);
void strAllToUpper(string &s);
void strAllToLower(string &s);
string ltrim(string s);
string rtrim(string s);
string trim(string s);
vector<string> splitTokens(string &s, const string &spr);
string format(int v, int len, char filc);
string strListIdx(vector<int> &lst);

/*----- wrapper functions for LAPACK -----------------------------------*/

#ifdef _WIN32
extern "C" {
  void DSYEV(char &jobz, char &uplo, int &n, double **a, int &lda,
   double *w, double *work, int &lwork, int &info);
}
#else
extern "C" {
  void dsyev_(char &jobz, char &uplo, int &n, double **a, int &lda,
   double *w, double *work, int &lwork, int &info);
}
#endif

int lapack_dsyev(char jobz, char uplo, int n, double **a, int lda,
 double *w, double *work, int lwork);

#endif

