/*========================================================================
     "tstrate"  TST rate constant and equilibrium const. calculation

             Copyright (c) 2002-2005 by A. Miyoshi, Univ. Tokyo
                                     created: Dec.  25, 2004
                                      edited: Jan.  13, 2005
                                      edited: Nov.  19, 2007
--------------------------------------------------------------------------
 usage: tstrate basename
--------------------------------------------------------------------------
 - Read basename.rxn file. Also requires .gpo (or .tst) files specified
    in the .rxn file.
 - Keywords allowed in .rxn file are:
    fileType ftyp(s)       ! "gpo" (default) or "tst"
    energyUnit uStr(s)     ! "hartree" (default), "kJ/mol", or "cm-1"
    tempRange Tstart(double) Tend(double) Tstep(double)
    tempList T1(double) T2(double) ...
    reactants{
      file f1Re(s) [f2Re(s) ...]
      energy eRe(double)
    }
    transitionState{
      file fTS(s)
      energy eTS(double)
      imagFreqTS imfrq(double)
    }
    products{
      file f1Pr(s) [f2Pr(s) ...]
      energy ePr(double)
    }
========================================================================*/

#include "gpoplibt.h"

/*===== main function ==================================================*/

int main(int argc, char *argv[]) {
  string basfln;
  reactionInfo rxninf;

  showRevision();
  if (argc != 2) { cout << "usage: tstrate basename\n"; return 1; }
  basfln = argv[1];
  if (rxninf.readFrom(basfln) == ERRE)
    { cout << "Error reading [" << basfln << ".rxn].\n"; return 1; }
  if (rxninf.readProp() == ERRE)
    { cout << "Error reading properties.\n"; return 1; }
  rxninf.calculate();
  rxninf.fitArrhenius();
  rxninf.outfileCSV();
  if (rxninf.calEck) {
    cout << "Maximum numerical error = " << rxninf.tnlC.maxNumError
         << endl;
  }
  return 0;
}

